/*
    RC522 I2C Module annonces gare

    www.locoduino.org - fev 2019, vers : v 0.6

  MFRC522.cpp - Library to use ARDUINO RFID MODULE KIT 13.56 MHZ WITH TAGS I2C BY AROZCAN
  arozcan @ https://github.com/arozcan/MFRC522-I2C-Library
  MFRC522.cpp - Based on ARDUINO RFID MODULE KIT 13.56 MHZ WITH TAGS SPI Library BY COOQROBOT
  miguelbalboa @ https://github.com/miguelbalboa/rfid

  Mapping @ http://www.locoduino.org/local/cache-vignettes/L610xH886/rc522_annonces_gare_0-87805.jpg?1552162449
  Si vous utilisez une carte Arduino Mega, SDA est sur le connecteur 20 et SCL sur le connecteur 21.
*/


#define DEBUG                           // Ne pas oublier de commenter en production

//-------------------------------- Module son -----------------------------------//

#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"       // https://github.com/DFRobot/DFRobotDFPlayerMini
const uint8_t soundVolume = 20;        // Sur 30 volume maxi

SoftwareSerial mySoftwareSerial(10, 11); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

// Plus d'infos sur DFPlayer : https://www.dfrobot.com/wiki/index.php/DFPlayer_Mini_SKU:DFR0299#Connection_Diagram

//-------------------------------- RC522 -----------------------------------//

#include <Wire.h>
#include "MFRC522_I2C.h"              // https://github.com/arozcan/MFRC522-I2C-Library

const uint8_t rstPin = 6;             // Pin pour RST
MFRC522 mfrc522(0x28, rstPin);        // Instance de MFRC522.

//-------------------------- Variable globales -----------------------------//


const long rc522tempo = 1 * 10UL;  // rc522tempo entre 2 lectures => 1 * 10UL = 10/1000 sec
long prev;
String uidOld;

const uint8_t nbreLocos = 3;          //Remplacer par le nombre de machines. Doit etre < 255

/*
 * Structure permettant de décrire une locomotive 
 */
typedef struct {
  uint8_t index; /* a quoi cela sert-il ? */
  char *  tagRFID; 
  char *  nom;
  char *  identifiant;
} Locomotive;

const Locomotive locosTab[nbreLocos] = {
  {0, "04263722665D81", "YC 6400   ", 0001},
  {1, "04463722665D81", "Renault VH", 0002},
  {2, "04726922665D80", "CC-7126   ", 0003}
};

//-------------------------- Definition des fonctions -----------------------------//

void DFRobotDFPlayerMini_printDetail(uint8_t type, int value);
void mfrc522_printDetail(void);
void DFRobotDFPlayerMini_printDetail(uint8_t type, int value);


//-------------------------------- setup -----------------------------------//

void setup() {
  Serial.begin(115200);                      // Initialise la communication serie avec le PC

  // SoftwareSerial
  mySoftwareSerial.begin(9600);              // Initialise mySoftwareSerial
  Serial.print(F("\nDFRobot DFPlayer Mini Demo\n"));
  Serial.print(F("Initialisation DFPlayer ... (Peut prendre 3~5 secondes)\n"));

  if (!myDFPlayer.begin(mySoftwareSerial)) {  // Use softwareSerial to communicate with mp3.
    Serial.print(F("Incapable de commencer :\n"));
    Serial.print(F("1.Verifier la connexion SVP !\n"));
    Serial.print(F("2.Inserez la carte SD SVP !\n"));
  }
  Serial.print(F("DFPlayer Mini online.\n"));
  myDFPlayer.volume(soundVolume);            //Règle le volume. De 0 to 30.


  Wire.begin();                              // Initialise I2C
  mfrc522.PCD_Init();                        // Initialise MFRC522
  mfrc522_printDetail();                     // Affiche des details sur le lecteur de carte MFRC522
  Serial.print(F("\n"));

}

//-------------------------------- loop -----------------------------------//

void loop() {

  mfrc522update(); // MAJ du RC522

  //Message detaille sur DFPlayer donnant differents etats ou erreurs.
  if (myDFPlayer.available()) {
    DFRobotDFPlayerMini_printDetail(myDFPlayer.readType(), myDFPlayer.read());
  }
}

//-------------------------------- fonctions -----------------------------------//


//////////////////////////////////////////////////////////////////////////////////
//                                mfrc522update
//////////////////////////////////////////////////////////////////////////////////

void mfrc522update() {

  while ( ! mfrc522.PICC_IsNewCardPresent() || ! mfrc522.PICC_ReadCardSerial() ) {
    delay(50);
  }

  if (millis() > prev + rc522tempo ) {            // Verifie que la rc522tempo est depassee
    String uidNew;
    for (uint8_t i = 0; i < mfrc522.uid.size ; i++) {
      uidNew += mfrc522.uid.uidByte[i] < 0x10 ? "0" : "";
      uidNew += String(mfrc522.uid.uidByte[i], HEX);
    }

    uidNew.toUpperCase();            // On met toutes les lettres du tag en majuscules

#ifdef DEBUG
    Serial.println(uidNew);          // Permet de lire n'importe quel tag passe sur le lecteur
                                     // et d'afficher son UID
#endif

    if (uidNew != uidOld) {          // Le tag est il different ?
      for (uint8_t i = 0; i < nbreLocos ; i++) {  // Recherche dans le tableau des locos
        if (uidNew == locosTab[i].tagRFID) {

#ifdef DEBUG
          Serial.print(F("Train : \t"));
          Serial.print(locosTab[i].nom);
          Serial.print(F("\n"));
#endif

          myDFPlayer.playMp3Folder(locosTab[i].identifiant);

          uidOld = uidNew;  // On met a jour la variable uidOld

          break;   // Une loco a ete trouvee, on quitte la boucle de recherche
        }
      }
    }
    prev = millis();       // On affecte a la variable prev l'heure courante
  }
}



//////////////////////////////////////////////////////////////////////////////////
//                                mfrc522_printDetail
//////////////////////////////////////////////////////////////////////////////////

void mfrc522_printDetail() {
  // Donne la version du logiciel MFRC522
  byte v = mfrc522.PCD_ReadRegister(mfrc522.VersionReg);
  Serial.print(F("MFRC522 Version du logiciel : 0x"));
  Serial.print(v, HEX);
  if (v == 0x91)
    Serial.print(F(" = v1.0"));
  else if (v == 0x92)
    Serial.print(F(" = v2.0"));
  else
    Serial.print(F(" (inconnue)"));
  Serial.println("");
  // Quand 0x00 ou 0xFF est retourne, la communication a probablement echouee
  if ((v == 0x00) || (v == 0xFF)) {
    Serial.println(F("ATTENTION : Echec de la communication, est-ce que le MFRC522 est correctement connecte ?"));
  }
}


//////////////////////////////////////////////////////////////////////////////////
//                         DFRobotDFPlayerMini_printDetail
//////////////////////////////////////////////////////////////////////////////////

void DFRobotDFPlayerMini_printDetail(uint8_t type, int value) {
  switch (type) {
    case TimeOut:
      Serial.println(F("Depassement du temps !"));
      break;
    case WrongStack:
      Serial.println(F("Pile erronee !"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Carte inseree !"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Carte enlevee !"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Carte en ligne !"));
      break;
    case DFPlayerUSBInserted:
      Serial.println("USB Insere !");
      break;
    case DFPlayerUSBRemoved:
      Serial.println("USB Retire !");
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Nombre :"));
      Serial.print(value);
      Serial.println(F(" Morceau termine !"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayer Erreur :"));
      switch (value) {
        case Busy:
          Serial.println(F("Carte non trouvee"));
          break;
        case Sleeping:
          Serial.println(F("Mode veille"));
          break;
        case SerialWrongStack:
          Serial.println(F("Pile erronee"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Somme de controle ne correspond pas"));
          break;
        case FileIndexOut:
          Serial.println(F("Index de fichier hors limite"));
          break;
        case FileMismatch:
          Serial.println(F("Fichier introuvable"));
          break;
        case Advertise:
          Serial.println(F("En annonce"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}

//////////////////////////////////////////////////////////////////////////////////
//                                        End
//////////////////////////////////////////////////////////////////////////////////
